/* Copyright 2005 The Apache Software Foundation or its licensors, as
 * applicable.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "ap_config.h"
#include "httpd.h"

#include "apr.h"
#include "apr_hooks.h"
#include "apr_optional_hooks.h"

typedef struct smtpd_auth_lookup_rec {
    apr_pool_t *pool;

    const char *user;
    apr_size_t  user_len;

    const char *pass;
    apr_size_t  pass_len;

} smtpd_auth_lookup_rec;

#if !defined(WIN32)
#define SMTPD_AUTH_DECLARE(type)            type
#define SMTPD_AUTH_DECLARE_NONSTD(type)     type
#define SMTPD_AUTH_DECLARE_DATA
#elif defined(SMTPD_AUTH_DECLARE_STATIC)
#define SMTPD_AUTH_DECLARE(type)            type __stdcall
#define SMTPD_AUTH_DECLARE_NONSTD(type)     type
#define SMTPD_AUTH_DECLARE_DATA
#elif defined(SMTPD_AUTH_DECLARE_EXPORT)
#define SMTPD_AUTH_DECLARE(type)            __declspec(dllexport) type __stdcall
#define SMTPD_AUTH_DECLARE_NONSTD(type)     __declspec(dllexport) type
#define SMTPD_AUTH_DECLARE_DATA             __declspec(dllexport)
#else
#define SMTPD_AUTH_DECLARE(type)            __declspec(dllimport) type __stdcall
#define SMTPD_AUTH_DECLARE_NONSTD(type)     __declspec(dllimport) type
#define SMTPD_AUTH_DECLARE_DATA             __declspec(dllimport)
#endif

APR_DECLARE_EXTERNAL_HOOK(smtpd_auth, SMTPD_AUTH, int, password, (smtpd_conn_rec *scr, smtpd_auth_lookup_rec *lookup_rec));


#define SMTPD_AUTH_METHOD_PROTOTYPE smtpd_conn_rec *scr, char *buffer, \
smtpd_return_data *in, void *data

#define SMTP_AUTH_METHOD_FUNC(name)  smtpd_auth_method_##name
#define SMTP_AUTH_METHOD_DECLARE(name) SMTPD_DECLARE(int) SMTP_AUTH_METHOD_FUNC(name)(SMTPD_AUTH_METHOD_PROTOTYPE)

typedef SMTPD_AUTH_DECLARE(int) smtpd_auth_method(SMTPD_AUTH_METHOD_PROTOTYPE);

typedef struct {
    smtpd_auth_method *func;
} smtpd_auth_method_st;

void smtpd_auth_register_method(smtpd_conn_rec *scr, char *key, smtpd_auth_method *func);
